#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import sys, time, urlparse, optparse
import ZSI
from twisted.python import log
from twisted.internet import selectreactor
from twisted.internet import reactor
import pyGridWare.generated.stubs.counterservice as COUNTER
from pyGridWare.utility.scripts.client import GetOptParser, GetPortKWArgs, SetUp
from pyGridWare.faults import BaseFault


############################################################################
# Counter
#     add
#     createCounter
############################################################################
def create(port):
    request = COUNTER.CreateCounterRequest()
    msg = port.createCounter(request)
    epr = "\tAddress -- %s\tRefProp -- %s\t\tany -- %s" \
        %(msg._EndpointReference._Address,
          msg._EndpointReference._ReferenceProperties,
          msg._EndpointReference._ReferenceProperties._any)
    log.msg("----- create -----\n%s" %epr)
    return msg

def add(port, value=999):
    log.msg("----- add ----- ")
    request = COUNTER.AddInputMessage(value)
    msg = port.add(request)
    log.msg("RESPONSE: ", msg)


############################################################################
# ResourceLifetime
#     Destroy
#     GetCurrentMessage
#     SetTerminationTime
############################################################################
def Destroy(port):
    log.msg("----- destroy ----- ")
    request = COUNTER.DestroyRequest()
    msg = port.Destroy(request)
    log.msg("RESPONSE: ", msg)

def SetTerminationTime(port, interval):
    log.msg("----- SetTerminationTime ----- ")
    request = COUNTER.SetTerminationTimeRequest()
    request._RequestedTerminationTime = time.gmtime(time.time()+interval)
    msg = port.SetTerminationTime(request)
    log.msg("TerminationTime: ", msg._NewTerminationTime)
    log.msg("CurrrentTime:    ", msg._CurrentTime)


############################################################################
# ResourceProperties
#     GetResourceProperty
############################################################################
def GetResourceProperty(port):
    log.msg("----- GetResourceProperty ----- ")
    request = COUNTER.GetResourcePropertyRequest(("http://counter.com", "Value"))
    msg = port.GetResourceProperty(request)
    log.msg("RESPONSE: ", msg._any)


def main():
    op = GetOptParser('CounterService')
    (options, args) = op.parse_args()
    SetUp(options)
    kw = GetPortKWArgs(options)

    locator = COUNTER.CounterServiceLocator()
    port = locator.getCounterPortType(**kw)
    response = create(port)

    kw['endPointReference'] = response._EndpointReference
    port = locator.getCounterPortType(**kw)

    add(port, 11)
    GetResourceProperty(port)
    add(port, 13)
    Destroy(port)
    try:
        add(port, 11)
    except (BaseFault, ZSI.fault.Fault), ex:
        log.msg("TERMINATED: Operation Failed after resource was destroyed.")
        log.msg("EXCEPTION: %s" %ex)
    else:
        log.msg("RESOURCE FAILED TO TERMINATE")

    reactor.stop()
 
if __name__ == '__main__':
    reactor.callWhenRunning(main)
    reactor.run()

