############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, socket, warnings
from ConfigParser import ConfigParser
from twisted.application import service, internet
from twisted.web.server import Site
from twisted.web.resource import Resource

###########################################################################
# Grab configuration information
#
###########################################################################
CP = ConfigParser()
CP.read('server-config.txt')
PORT = CP.getint('Location', 'port')
HOST = socket.gethostbyname(socket.gethostname())
DEBUG = CP.getint('logging', 'debug')
SIGN_NOTIFY = CP.getint('WSSecurity', 'notifications')
USE_SSL = CP.getint('security', 'ssl')
CERTFILE = CP.get('security', 'certfile') or '%s/%s' %(os.environ['HOME'],'.globus/usercert.pem')
KEYFILE = CP.get('security', 'keyfile') or '%s/%s' %(os.environ['HOME'],'.globus/userkey.pem')
CALIST = filter(lambda s: s, map(lambda s: s.strip(), CP.get('security', 'ca').split(' ')))
AUTHSTYLE = CP.get('security', 'authorization')
ACL = CP.get('authorization', AUTHSTYLE)


###########################################################################
# Debugging -- if config debug is 0, turn off logging.
#
###########################################################################
if DEBUG:
    from ZSI.wstools.logging import setBasicLoggerDEBUG
    setBasicLoggerDEBUG()
else:
    from twisted.python.log import removeObserver, theLogPublisher
    for ob in theLogPublisher.observers: 
        removeObserver(ob)


###########################################################################
# pyGridWare imports
#
###########################################################################
from pyGridWare.generated.types import *
from pyGridWare.services.CounterService import Counter
from pyGridWare.services.PerfCounterService import Counter as PCounter
from pyGridWare.services.SubscriptionManager import SubscriptionManager
from pyGridWare.utility.http import ChunkedHTTPChannel
from pyGridWare.wsrf.resource import FileServerResource

SECURE_MESSAGE=True
try:
    from pyGridWare.services.SecureConversationService import SecureConversationService
    from pyGridWare.services.SecureCounterService import SecureCounter
    from pyGridWare.services.SecurePerfCounterService import Counter as PSCounter
except ImportError,ex:
    SECURE_MESSAGE=False

###########################################################################
# Get Resources
#
###########################################################################
def GetResource(contextFactory=None):
    root = Resource()
    root.putChild('wsdl', FileServerResource('wsdl'))
    root.putChild('wsrf', Resource())

    resource = Resource()
    root.getStaticEntity('wsrf').putChild('services', resource)

    resource.putChild('CounterService', 
        Counter(post='/wsrf/services/CounterService')
    )
    resource.putChild('PerfCounterService', 
        PCounter(post='/wsrf/services/PerfCounterService')
    )
    if SECURE_MESSAGE is True:
        resource.putChild('SecureCounterService-authService', 
            SecureConversationService(post='/wsrf/services/SecureCounterService-authService')
        )
        resource.putChild('SecureCounterService', 
            SecureCounter(post='/wsrf/services/SecureCounterService')
        )
        resource.putChild('SecurePerfCounterService', 
            PSCounter(post='/wsrf/services/SecurePerfCounterService')
        )

    # 
    # Need to Set URL for SubscriptionManager, need to know
    # its URL before it has received a request.
    # 
    post = '/wsrf/services/SubscriptionManagerService'
    scheme = 'http'
    if USE_SSL:
        scheme = 'https'

    subResource = SubscriptionManager.getInstance(post=post)
    subResource.setServiceURL('%s://%s:%d%s' %(scheme,HOST,PORT,post))
    resource.putChild('SubscriptionManagerService', subResource)
    return root


###########################################################################
# Set up Application
#
###########################################################################
Site.protocol = ChunkedHTTPChannel
myFactory = None
port = None

if SIGN_NOTIFY and SECURE_MESSAGE is True:
    from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler
    SubscriptionManager.kw['sig_handler'] = GssSignatureHandler()
    SubscriptionManager.kw['sig_handler'].setSecureMessage()

if USE_SSL:
    from OpenSSL import SSL
    from pyGridWare.utility import ssl 
    kw = {'sslmethod':SSL.SSLv3_METHOD, 'calist':CALIST}
    ContextFactoryClass = ssl.ServerContextFactory
    if AUTHSTYLE == 'acl':
        kw['aclfile'] = ACL
        ContextFactoryClass = ssl.ACLServerContextFactory
    elif AUTHSTYLE == 'gridmap':
        kw['aclfile'] = ACL
        ContextFactoryClass = ssl.GridMapServerContextFactory

    ctxFactory = ContextFactoryClass(KEYFILE, CERTFILE, **kw)
    SubscriptionManager.kw['contextFactory'] = ctxFactory
    port = internet.SSLServer(PORT, myFactory, ctxFactory)
else:
    port = internet.TCPServer(PORT, myFactory)

# Guess I need to hack server in like so...
Resource.server = port
port.args = list(port.args)
port.args[1] = Site(GetResource())
port.args = tuple(port.args)

application = service.Application("WSRF-Container")
port.setServiceParent(application)

