############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
__all__ = []
import ZSI as _ZSI
from ZSI.TC import _get_global_element_declaration as _GED
from ZSI.TC import _get_type_definition as _GTD
from ZSI.TC import ElementDeclaration as _ElementDeclaration
from ZSI.TC import TypeDefinition as _TypeDefinition

"""Patches for secure conversation not generated <choice> model
groups correctly.  This module needs to be imported before 
secure conversation types in order for it to be used.

Also need to import to interop correctly with Globus
"""

class ReferenceParametersType_Def(_ZSI.TC.Struct, _TypeDefinition):
    '''BUG This type is not defined in ws-addressing, but it's being used
    by Globus.
    '''
    #<wsa:ReferenceParameters xsi:type="wsa:ReferenceParametersType"/>
    schema = 'http://schemas.xmlsoap.org/ws/2004/03/addressing'
    type = (schema, "ReferenceParametersType")

    def __init__(self, pname, **kw):
        class Holder:
            typecode = self
        _ZSI.TC.Struct.__init__(self, Holder, [], pname=pname, inorder=0, **kw)


class SetResourceProperties_Dec(_ZSI.TCcompound.Struct, _ElementDeclaration):
    '''Patch for <choice>
    '''
    schema = "http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd"
    literal = "SetResourceProperties"
    def __init__(self, **kw):
        ns = SetResourceProperties_Dec.schema
        ens = "http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd"
        TClist = [_GED(ens,"Insert")(minOccurs=0, maxOccurs=1, nillable=0), 
            _GED(ens,"Update")(minOccurs=0, maxOccurs=1, nillable=0), 
            _GED(ens,"Delete")(minOccurs=0, maxOccurs=1, nillable=0),
        ]
        kw["pname"] = ("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd","SetResourceProperties")
        kw["aname"] = "_SetResourceProperties"
        class Holder:
            typecode = self
            def __init__(self):
                # pyclass
                self._Insert = None
                self._Update = None
                self._Delete = None
                return
        _ZSI.TCcompound.Struct.__init__(self, Holder, TClist, inorder=0, **kw)


class KeyInfoType_Def(_ZSI.TCcompound.Struct, _TypeDefinition):
    '''Patch for <choice>
    '''
    schema = "http://www.w3.org/2000/09/xmldsig#"
    type = (schema, "KeyInfoType")
    attribute_typecode_dict = {}
    def __init__(self, pname, **kw):
        TClist = [\
            _GED(self.schema,'KeyName')(minOccurs=0, maxOccurs=1, nillable=0),
            _GED(self.schema,'KeyValue')(minOccurs=0, maxOccurs=1, nillable=0),
            _GED(self.schema,'RetrievalMethod')(minOccurs=0, maxOccurs=1, nillable=0),
            _GED(self.schema,'X509Data')(minOccurs=0, maxOccurs=1, nillable=0),
            _GED(self.schema,'PGPData')(minOccurs=0, maxOccurs=1, nillable=0),
            _GED(self.schema,'SPKIData')(minOccurs=0, maxOccurs=1, nillable=0),
            _GED(self.schema,'MgmtData')(minOccurs=0, maxOccurs=1, nillable=0),
            _ZSI.TC.AnyElement(aname="_any", minOccurs=0, maxOccurs=1, nillable=0)]
        # attribute handling code
        self.attribute_typecode_dict.update(_GTD(self.schema,'KeyInfoType').attribute_typecode_dict)
        self.attribute_typecode_dict["Id"] = _ZSI.TC.AnyType()

        class Holder:
            typecode = self
            def __init__(self):
                # pyclass
                self._attrs = {}
                self._KeyName = None
                self._KeyValue = None
                self._RetrievalMethod = None
                self._X509Data = None
                self._PGPData = None
                self._SPKIData = None
                self._MgmtData = None

        _ZSI.TCcompound.Struct.__init__(self, Holder, TClist,\
            pname=pname, inorder=0, mixed=True, mixed_aname="_text", **kw
        )


