############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import time
from ZSI.TC import _get_type_definition as GTD
from ZSI.wstools.Namespaces import OASIS
from twisted.internet import reactor
from twisted.python import log
from pyGridWare.addressing.AddressingUtils import AddressingUtils
from pyGridWare.properties.ResourceProperties import NotificationProducerRP
from pyGridWare.generated.stubs.basenotification import \
        WS_BaseNotificationServiceLocator, Notify

import pyGridWare.generated.types.basenotification
import pyGridWare.generated.types.notificationconsumerservice

'''
'''

_MESSAGE = GTD("http://wsrf.globus.org/core/notification",
    "ResourcePropertyValueChangeNotificationElementType")(pname='Message')
_RPV = filter(\
    lambda w: (w.nspname,w.pname) == (OASIS.PROPERTIES,'ResourcePropertyValueChangeNotification'),
    _MESSAGE.ofwhat,
    )[0]
_NEWVALUE = filter(\
    lambda w: (w.nspname,w.pname) == (OASIS.PROPERTIES,'NewValue'),
    _RPV.ofwhat, 
    )[0]

def _GetMessage(property):
    pyobj = _MESSAGE.pyclass()
    rpv = pyobj._ResourcePropertyValueChangeNotification = _RPV.pyclass()
    rpv._NewValue = _NEWVALUE.pyclass()
    rpv._NewValue._any = property
    return pyobj

class _Holder: 
    pass


class NotificationProducer:
    '''The NotificationProducer.
    '''
    def AddProducer(cls, subscription, **kw):
        '''Use subscription to add a topic listener for the specified 
        producer property.

        subscription -- 
        request -- ConsumerReference element declaration, contents
            ConsumerReference, TopicExpression, UseNotify
        '''
        producer = subscription.producer
        properties = producer.properties 
        request = subscription.request
        if isinstance(properties, NotificationProducerRP) is False:
            raise SubscribeCreationFailedFault(timestamp=time.gmtime(),
                faultcause=('Invalid Notification Producer Properties.'),
            )

        url = subscription.reference._Address
        locator = WS_BaseNotificationServiceLocator()
        stub = locator.getNotificationConsumer(url=url, \
            endPointReference=request._ConsumerReference, **kw)

        if request._UseNotify is True:
            properties.addTopicListener(cls.Notify, subscription, stub)
        else:
            log.warn('Subscription Request UseNotify False is Not Supported')

    AddProducer = classmethod(AddProducer)

    def Notify(property, sub, stub):
        '''Send Notify message. Called by TopicListener.
        sub -- Subscription
        stub -- Notification Producer Stub
        '''
        pause = sub.context.SubscriptionPolicy
        if pause == False:
            return
        if pause != True:
            raise RuntimeError, 'SubscriptionPolicy expected to be True/False, not %s'\
                %sub.context.SubscriptionPolicy

        log.debug("Send Notification -->")
        request = Notify()
        NotificationMessage = _Holder()
        request._NotificationMessage = (NotificationMessage,)
        NotificationMessage._Topic = sub.topic
        NotificationMessage._ProducerReference = \
            AddressingUtils.createEndpointReference(sub.producer)
        NotificationMessage._Message = _GetMessage(property)
        stub.Notify(request)
    Notify = staticmethod(Notify)


