############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
from ZSI.TC import _get_global_element_declaration as GED
from twisted.python import log
from pyGridWare.patterns import Singleton
from pyGridWare.addressing.AddressingUtils import AddressingUtils
from pyGridWare.notification.NotificationProducer import NotificationProducer
from pyGridWare.resource.ResourceContext import ResourceContext
from pyGridWare.lifetime.ResourceLifetime import ImmediateResourceTermination,\
    ScheduledResourceTermination

import pyGridWare.generated.types.basenotification
from pyGridWare.generated.stubs.basenotification import \
    WS_BaseNotificationServiceLocator, NotificationConsumerSOAPBindingSOAP

from pyGridWare.generated.resource.submanagerservice.SubscriptionManagerService import \
    SubscriptionManagerContext


class Subscription(object):
    '''Represents a subscription, registered to NotificationProducer
    '''
    def __init__(self, request, context, producer):
        '''Instance Data:
        request  -- Subscription Request Message.
        context  -- SubscriptionManagerContext instance.
        producer -- Monitoring Resource Context. 
        '''
        self.request = request
        self.context = context
        self.producer = producer

    def getContext(self):
        return self.__sub_context
    def setContext(self, context):
        assert isinstance(context, SubscriptionManagerContext), \
            'resource must be a SubscriptionManagerContext instance'
        self.__sub_context = context
    context = property(getContext, setContext, None,
        'Subscription Context')
 
    def getProducerContext(self):
        return self.__p_context
    def setProducerContext(self, context):
        assert isinstance(context, ResourceContext), \
            'resource must be a ResourceContext instance'
        self.__p_context = context
    producer = property(getProducerContext, setProducerContext, None,
        'Subscribed to Resource Context')

    def getSubscriptionRequest(self):
        return self.__request
    def setSubscriptionRequest(self, request):
        self.__request = request
    request = property(getSubscriptionRequest, setSubscriptionRequest, None,
        'Subscription Request')

    def getTopicExpression(self):
        return self.request._TopicExpression
    topic = property(getTopicExpression, None, None,
        'TopicExpression')

    def getConsumerReference(self):
        return self.request._ConsumerReference
    reference = property(getConsumerReference, None, None,
            'ConsumerReference')

