############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
from pyGridWare.generated.services.printerservice.PrinterService import \
    PrinterServiceWSRF as WSRFService

"""
Printer object - Physical printer or printer server
 Operations:
     Print-Job – submit a document for printing – document included with the request.
     Print-URI – submit a document for printing – document identified by the URI.
     Validate-Job – Check print attributes without actually printing.
     Create-Job – Create a job capable of holding multiple documents. 
     Get-Printer-Attributes – Find out about the printer
     Get-Jobs – Get a list of the jobs queued for the printer.

Job object - used to model a print job.  A Job object contains documents.
  Operations:
     Send-Document – Add to a multi-document job – document included with the request.
     Send-URI - Add to a multi-document job  – document identified by the URI.
     Cancel-Job
     Get-Job-Attributes

<wsrf-pr:CreatePrinterResponse">
  <wsrf-pr:printer_reference>
    <wsa:Address>http://www.WSRF-Examples.org/IPPprinters</wsa:Address>
    <wsa:ReferenceProperties>
      <name>Printer1</name>
    </wsa:ReferenceProperties>
    <wsa:PortType>wsrf-prw:Printer</wsa:PortType >
  </wsrf-pr:printer_reference>
</wsrf-pr:CreatePrinterResponse>

"""

class PrinterService(WSRFService):
    def __init__(self, post='/PrinterFactory', **kw):
        WSRFService.__init__(self, post)

    def wsa_Print_Job(self, ps, address):
        response = WSRFIPP.wsa_Print_Job(self, ps, address)
        ctx = WSRFService.GetResourceContext(ps, address)
        return response

    def wsa_Create_Job(self, ps, address):
        response = WSRFIPP.wsa_Create_Job(self, ps, address)
        ctx = WSRFService.GetResourceContext(ps, address)
        return response

    def wsa_Send_Document(self, ps, address):
        response = WSRFIPP.wsa_Send_Document(self, ps, address)
        ctx = WSRFService.GetResourceContext(ps, address)
        return response

    def wsa_CreatePrinter(self, ps, address):
        response = WSRFIPP.wsa_CreatePrinter(self, ps, address)
        #ctx = WSRFService.GetResourceContext(ps, address)
        return response
