############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
from twisted.python import log
import ZSI
from ZSI.wstools.Namespaces import WSA200403

import pyGridWare.generated.types.basenotification
from pyGridWare.generated.resource.submanagerservice.SubscriptionManagerService import \
     SubscriptionManagerContext
from pyGridWare.generated.services.submanagerservice.SubscriptionManagerService import \
    SubscriptionManagerServiceWSRF

from pyGridWare.patterns import Singleton
from pyGridWare.addressing.AddressingUtils import AddressingUtils
from pyGridWare.resource.ResourceContext import ResourceContext
from pyGridWare.lifetime.ResourceLifetime import ImmediateResourceTermination,\
    ScheduledResourceTermination
from pyGridWare.notification.SubscriptionManager import Subscription
from pyGridWare.notification.NotificationProducer import NotificationProducer


class Policy(int):
    typecode = ZSI.TC.Boolean('pause')


class SubscriptionManager(SubscriptionManagerServiceWSRF, Singleton):
    '''class varialbes:
    subscriptions -- dict of Subscription instances.
    '''
    subscriptions = {}
    kw = {\
        'wsAddressURI':WSA200403.ADDRESS,
    }
    def __init__(self, post='/wsrf/services/SubscriptionManagerService', **kw):
        SubscriptionManagerServiceWSRF.__init__(self, post, **kw)

    def subscribe(self, request, rctx):
        '''Returns an EPR representing a SubscriptionReference.
        request -- subscription request, containing ConsumerReference and TopicExpression.
        rctx -- resource context subscription applies to.
        '''
        assert isinstance(rctx, ResourceContext), \
            'resource must be a ResourceContext instance'

        # SubscriptionManager Context
        ctx = self.NewResourceContextSubscriptionManagerPort()
        ctx.SubscriptionPolicy = Policy(1)
        sub = Subscription(request, ctx, rctx)

        NotificationProducer.AddProducer(sub, **SubscriptionManager.kw)
        SubscriptionManager.subscriptions[id(ctx)] = sub

        epr = AddressingUtils.createEndpointReference(ctx)
        return epr

    def wsa_PauseSubscription(self, ps, address):
        # How to store pause state?
        ctx = self.GetResourceContext(ps, address)
        ctx.SubscriptionPolicy = Policy(0)
        response = SubscriptionManagerServiceWSRF.wsa_PauseSubscription(self, ps, address)

        # Key Subscription instance w/id of subscription context instance
        sub = SubscriptionManager.subscriptions[id(ctx)]
        return response

    def wsa_ResumeSubscription(self, ps, address):
        # How to store pause state?
        ctx = self.GetResourceContext(ps, address)
        ctx.SubscriptionPolicy = Policy(1)
        response = SubscriptionManagerServiceWSRF.wsa_ResumeSubscription(self, ps, address)

        # Key Subscription instance w/id of subscription context instance
        sub = SubscriptionManager.subscriptions[id(ctx)]
        return response


